import path from "path";
import process from "process";
import * as fs from "fs";
import {Config, GameRules} from "./RawConstants";
import {app} from "electron";


export class ConfigHandler {
    private static dataPath = path.resolve(app.getPath('userData'), "config");
    private static confPath = path.resolve(this.dataPath, "config.json");

    private static defaultConfig = {
        rules: {
            switchValues: {},
            inputValues: {}
        }
    }


    static generateDefault() {
        fs.writeFileSync(this.confPath, JSON.stringify(this.defaultConfig, null, 1));
    }

    static get() {
        this.existsOrGenerate();

        let confRaw = fs.readFileSync(this.confPath).toString('utf8');

        try {
            JSON.parse(confRaw);
        } catch (e) {
            this.generateDefault();
        }

        console.log("Config retrieved!")
        return JSON.parse(confRaw) as Config;
    }

    static existsOrGenerate() {
        if (!fs.existsSync(this.dataPath))
            fs.mkdirSync(this.dataPath);

        if (!fs.existsSync(this.confPath))
            this.generateDefault();
    }

    static set(conf: Config) {
        this.existsOrGenerate();

        let confRaw = fs.readFileSync(this.confPath).toString('utf8');

        try {
            JSON.parse(confRaw);
        } catch (e) {
            this.generateDefault();
        }


        let current = JSON.parse(confRaw) as Config;
        current = {...current, ...conf};
        console.log("Config saved to " + this.confPath);
        fs.writeFileSync(this.confPath, JSON.stringify(current, null, 1));
    }


}