export type IPCChannel =
    'WIFI_SCAN'
    | 'WIFI_CONNECT'
    | 'FUNCTION_TEST'
    | 'CLOUD_STATUS'
    | 'CLOUD_CONNECT'
    | 'SETTINGS'
    | 'PREPARING'

export enum IPCListenChannels {
    'KEYPAD_INPUT' = 'KEYPAD_INPUT',
    'NFC_CARD' = 'NFC_CARD',
    'NFC_RAW' = 'NFC_RAW'
}

export interface IPCAnswer {
    status: boolean,
    data?: any
}

export interface IPCRequest {
    data?: any,
}

export interface WiFiNetwork {
    ssid: string,
    isSecured: boolean,
    isKnown?: boolean,
    psk?: string,
    rssi?: number,
}

export interface FunctionTest {
    hasSudo: boolean,
    hasWPASupplicant: boolean,
    hasInternet: boolean
}

export type GameSwitch = {
    name: GameSwitchNames,
    label: string,
    depends?: GameSwitchNames,
    value: boolean
}

export type GameInput = {
    name: GameInputNames,
    type: "number" | "text",
    label: string,
    depends?: GameSwitchNames,
    value: string
}

export type GameSwitchNames =
    "GET_BONUS_PASSING_START"
    | "DOUBLE_BONUS_ON_GO"
    | "PUT_PAID_PENALTIES_IN_MIDDLE"
    | "RECEIVE_PENALTIES_AT_FREE_PARKING"
    | "PUT_10_IN_MIDDLE_AFTER_EACH_GO"
    | "STREET_MUST_BE_AUCTIONED_IF_NOT_PURCHASED"
    | "TWO_HOTELS_PER_STREET_ALLOWED"
    | "MORTGAGE_REDUCES_RENT"
    | "GOODS_GO_TO_OTHER_PLAYERS_UPON_BANKRUPTCY"
    | "STREETS_CAN_BE_SOLD_BACK_TO_THE_BANK"
    | "PRISON_CAN_BE_PURCHASED_RENT_GOES_TO_PLAYER"
    | "KEEP_MONEY_MORE_SECRET";

export type GameInputNames =
    "PASSING_GO_CASH"
    | "STARTING_CASH"
    | "PRISON_RELEASE_FEE"
    | "CURRENCY";

export interface GameRules {
    switchValues: { [key in GameSwitchNames]?: boolean },
    inputValues: { [key in GameInputNames]?: string }
}

export interface Config {
    rules?: GameRules
}

export enum NFCCardType {
    INVALID = "INVALID",
    ACCOUNT = "ACCOUNT",
    PROPERTY = "PROPERTY",
    TASK = "TASK",
}

export interface NFCCard {
    cardType: NFCCardType,
    uid?: string,
    raw?: string
}

export enum AccountSymbols {
    HAT= "Hut",
    CAT= "Katze",
    SHOE= "Schuh",
    SHIP= "Kriegsschiff",
    RACE_CAR = "Rennauto",
    THIMBLE= "Fingerhut",
    WHEELBARROW= "Schubkarre",
    DOG = "Hund",
    IRON = "Bügeleisen",
    BATTLESHIP = "Schlachtschiff",
    TOP_HAT = "Zylinder",
    CANNON = "Kanone",
    TREX = "T-Rex",
    RUBBER_DUCK = "Gummiente",
    PENGUIN = "Pinguin",

    PLAYTYPUS= "Schnabeltier"
}

export interface NFCAccountCard extends NFCCard {
    symbol: AccountSymbols,
    nickname: string,
    pin: number
}

export enum PropertyColor {
    YELLOW= "YELLOW",
    GOLD="GOLD",
    SILVER="SILVER",
    ORANGE = "ORANGE",
    PINK = "PINK",
    BEIGE="BEIGE",
    BROWN = "BROWN",
    RED ="RED",
    GREEN="GREEN",
    BLACK="BLACK",
    WHITE="WHITE",
    LIGHT_BLUE ="LIGHT_BLUE",
    DARK_BLUE = "DARK_BLUE",

    TRAINSTATION = "TRAINSTATION",
    UTILITY = "UTILITY",

    CUSTOM1 ="CUSTOM1",
    CUSTOM2 = "CUSTOM2",
    CUSTOM3 = "CUSTOM3"
}

export type NFCPropertyCardSpecialProperties =
    "TRAINSTATION" | "UTILITY"

/**
 * NFC Property Card
 * NAME(str);COLOR(str);FULLSET(num);BUYVAL(num);MORTGAGEVAL(num);RENT,RENT1,RENT2,RENT3,RENT4,RENT_HOTEL,SPECIAl_PROPS
 */
export interface NFCPropertyCard extends NFCCard {
    name: string,
    color: PropertyColor,
    fullSetAmount: number,
    buyValue: number,
    mortgageValue: number,
    rent: number,
    rent1?: number,
    rent2?: number,
    rent3?: number,
    rent4?: number,
    rentHotel?: number,
    specialProperties: NFCPropertyCardSpecialProperties|"",
}

export enum TaskType {
    GET_FROM_BANK= "GET_FROM_BANK", // 1 val
    PAY_TO_BANK= "PAY_TO_BANK", // 1 val
    PAY_TO_ALL_PLAYERS= "PAY_TO_ALL_PLAYERS", // 1 val
    PAY_FOR_PROPS= "PAY_FOR_PROPS", // 2 val
    ESCAPE_JAIL= "ESCAPE_JAIL" // 0 val
}

/**
 * NFC Task Card
 * RAW: TYPE;AMOUNT;AMOUNT2
 */
export interface NFCTaskCard extends NFCCard {
    type: TaskType
    amount?: number,
    amount2?: number
}

