import {IPCHandle} from "./IPCMainHandler";
import {FunctionTest, GameRules} from "./RawConstants";
import OSHandler from "./OSHandler";
import CloudHandler from "./CloudHandler";
import {ConfigHandler} from "./ConfigHandler";
import NFCHandler from "./NFCHandler";

const wifiScan = require("node-wifi-scanner");


/**
 * Background Class for doing stuff on the host
 */
export default class SmartMonopoly {
    static run() {
        this.setupIPCEvents();
        OSHandler.enableAllWifis().then().catch(console.error);
        NFCHandler.initTest();
    }

    static setupIPCEvents() {
        IPCHandle("FUNCTION_TEST", async (e, request, args) => {
            let data: FunctionTest = {
                hasSudo: await OSHandler.checkForSudo(),
                hasWPASupplicant: false,
                hasInternet: await OSHandler.checkForInternet(),
            };

            return {
                status: true,
                data: data
            }
        });

        IPCHandle("WIFI_CONNECT", async (e, request, args) => {
            let data = request.data as { ssid: string, psk: string }
            try {
                await OSHandler.addWifi(data.ssid, data.psk);
                let status = await OSHandler.connectToWifi(data.ssid);
                return {status: status};
            } catch (e) {
                return {status: false}
            }
        });

        IPCHandle("WIFI_SCAN", async (e, request) => {
            try {
                let networks = await OSHandler.scanWifis();
                return {status: true, data: networks};
            } catch (e) {
                return {status: false};
            }
        });

        IPCHandle("CLOUD_CONNECT", async (e, request) => {
            try {
                await CloudHandler.connect();
                return {status: true};
            } catch (e) {
                return {status: false, data: e.toString()};
            }
        });

        IPCHandle("SETTINGS", async(e, request) => {
            if(request.data && request.data["rules"])
            {
                let rules = request.data.rules as GameRules;
                ConfigHandler.set({rules: rules});
            }

            return {status: true, data: ConfigHandler.get()};
        });

        IPCHandle("PREPARING", async(e, request) => {
            const useCloud = !!request.data.useCloud;
            console.log("Preparing - useCloud: " + useCloud);
            if(useCloud && !CloudHandler.isConnected())
                try {
                    await CloudHandler.connect();
                }catch(e)
                {
                    return {status: false, data: 'Cloud connection issue: ' + e};
                }

            //todo when using cloud, get session etc.

            return {status: true};
        })
    }
}