import {PAGE} from "./App";
import {IPCListenChannels, NFCCard} from "../RawConstants";
import IPCRendererListener from "./IPCRendererListener";

export enum GAME_STATE {
    NOT_STARTED,
    PREPARING,
    PLAYING,
    END
}

export default class GameHandler {

    static GAMESTATE: GAME_STATE = GAME_STATE.NOT_STARTED;
    static attachedNFCHandler: number = 0;

    static setGameState(state: GAME_STATE)
    {
        switch (state)
        {
            case GAME_STATE.NOT_STARTED:
                window.app.setPage(PAGE.STARTUP);
                break;
            case GAME_STATE.PREPARING :
                window.app.setPage(PAGE.GAME_SETUP);
                break;
        }

        this.GAMESTATE = state;
    }

    static attachNFCHandler(cb: (card: NFCCard) => void)
    {
        IPCRendererListener.detach(this.attachedNFCHandler);
        this.attachedNFCHandler = IPCRendererListener.attach(IPCListenChannels.NFC_CARD, (ipcMsg) => {
            if(ipcMsg.status)
                cb(ipcMsg.data as NFCCard);
        });
    }

    static detachNFCHandler()
    {
        IPCRendererListener.detach(this.attachedNFCHandler);
    }

    static async requestPreparing(useCloud: boolean)
    {
        let response = await window.api.request("PREPARING", {
            data:
                {
                    useCloud: useCloud,
                }
        });

        if(!response.status)
        {
            this.setGameState(GAME_STATE.NOT_STARTED);
            throw new Error(response.data);
        }

        this.setGameState(GAME_STATE.PREPARING);
        return true;
    }
}