import React, {Component} from "react";
import {
    Button, Dialog,
    DialogActions,
    DialogContent,
    DialogContentText,
    DialogTitle,
    Grid, ListItemAvatar,
    Snackbar,
    Typography,
    Avatar, ListItemText, ListItem, List, Chip
} from "@mui/material";
import NfcIcon from '@mui/icons-material/Nfc';
import AddCardIcon from '@mui/icons-material/AddCard';
import PersonAddIcon from '@mui/icons-material/PersonAdd';


import GameHandler from "./GameHandler";
import {NFCCard} from "../RawConstants";
import {Person as PersonIcon, QrCode, RemoveDone} from "@mui/icons-material";
import CloudIcon from "@mui/icons-material/Cloud";

interface GameState {
    snackMsg: string,
    helpDialog: boolean
}

export default class GameSetup extends Component<{}, GameState> {
    constructor(props: {}) {
        super(props);
        this.state = {
            snackMsg: "",
            helpDialog: true
        }
    }


    componentDidMount() {
        setTimeout(() => {
            this.setState((prevState) => ({
                ...prevState,
                helpDialog: false,
                snackMsg: "Zum hinzufügen von Spielern, NFC-Karte an das Lesegerät halten!",
            }))
        }, 15000);

        GameHandler.attachNFCHandler((card: NFCCard) => {

        })
    }

    componentWillUnmount() {
        GameHandler.detachNFCHandler();
    }

    render() {
        return <div className="gameSetup">
            <Snackbar
                open={!!this.state.snackMsg}
                autoHideDuration={10000}
                onClose={() => {
                    this.setState(prevState => ({
                        ...prevState,
                        snackMsg: ""
                    }))
                }}
                message={this.state.snackMsg}
            />
            <Dialog
                open={this.state.helpDialog}
                onClose={() => {
                    this.setState((prevState) => ({
                        ...prevState,
                        helpDialog: false
                    }))
                }}
                aria-labelledby="alert-dialog-title"
                aria-describedby="alert-dialog-description"
            >
                <DialogTitle id="alert-dialog-title">
                    Spielerkarte auf Gerät legen! <AddCardIcon/>
                </DialogTitle>
                <DialogContent>
                    <DialogContentText id="alert-dialog-description">
                        Um eine Figur zum Spiel hinzuzufügen, musst du die NFC-Spielerkarte an den Kartenleser
                        halten. <br/><br/>
                        Ein kurzes Berühren reicht aus, schon sollte der Spieler in der Tabelle auftauchen! <PersonAddIcon/>
                    </DialogContentText>
                </DialogContent>
                <DialogActions>
                    <Button onClick={() => this.setState((prevState) => ({
                        ...prevState,
                        helpDialog: false
                    }))}>
                        Okay</Button>
                </DialogActions>
            </Dialog>

            <Chip sx={{position: "absolute", top: "1%", right: "1%"}} onClick={() => {this.setState(prevState => ({
                ...prevState,
                helpDialog: true
            }))}}
                  label="?" color="info"
                  variant="outlined"/>

            <Typography variant="h2" >Wer spielt mit?</Typography>
            <Chip onClick={() => {this.setState(prevState => ({
                ...prevState,
                snackMsg: "Um einen Spieler hinzuzufügen, lege eine NFC-Karte auf den leser!"
            }))}}
                label={<div><NfcIcon sx={{marginTop: 0.9}} className={"animationBreathe"}
                                     color="primary"/> NFC</div>} color="secondary"
                variant="outlined"/>
            <Grid sx={{mt: 2}} container spacing={1}>
                <Grid item xs={2}></Grid>
                <Grid justifyContent="center" alignItems="center" item xs={"auto"}>
                    <List sx={{ width: '100%', minWidth: '60%', bgcolor: 'background.paper', border: "1px solid black", borderRadius: "5px" }}>
                        <ListItem>
                            <ListItemAvatar>
                                <Avatar>
                                    <PersonIcon />
                                </Avatar>
                            </ListItemAvatar>
                            <ListItemText primary="Hut" secondary="Connect+ QR-Code anzeigen" onClick={() => {console.log("connect?")}}/>
                            <Button sx={{ml: 4}} variant="outlined" color="info">Umbenennen</Button>
                            <Button sx={{ml: 1}} variant="outlined" color="error">Rauswerfen</Button>
                            <Button disabled sx={{ml: 1}} variant="outlined" color="primary"><QrCode/></Button>
                        </ListItem>
                        <ListItem>
                            <ListItemAvatar>
                                <Avatar>
                                    <PersonIcon />
                                </Avatar>
                            </ListItemAvatar>
                            <ListItemText primary="Schubkarre" secondary="Connect+ QR-Code anzeigen" onClick={() => {}}/>
                            <Button sx={{ml: 4}} variant="outlined" color="info">Umbenennen</Button>
                            <Button sx={{ml: 1}} variant="outlined" color="error">Rauswerfen</Button>
                            <Button disabled sx={{ml: 1}} variant="outlined" color="primary"><QrCode/></Button>
                        </ListItem>
                        <ListItem>
                            <ListItemAvatar>
                                <Avatar>
                                    <CloudIcon />
                                </Avatar>
                            </ListItemAvatar>
                            <ListItemText primary="Schuh" secondary="Connect+ verbunden" onClick={() => {}}/>
                            <Button sx={{ml: 4}} variant="outlined" color="info">Umbenennen</Button>
                            <Button sx={{ml: 1}} variant="outlined" color="error">Rauswerfen</Button>
                            <Button disabled sx={{ml: 1}} variant="outlined" color="primary"><QrCode/></Button>
                        </ListItem>
                    </List>
                </Grid>
            </Grid>
        </div>
    }

    }